@@ -30,14 +30,24 @@ CREATE INDEX IF NOT EXISTS idx_run_run_id ON RUN (ID);
 CREATE INDEX IF NOT EXISTS idx_feature_run_id ON FEATURE (RUN_ID);
 
 CREATE INDEX IF NOT EXISTS idx_feature_feature_id ON FEATURE (ID);
-CREATE INDEX IF NOT EXISTS idx_feature_ms1_feature_id ON FEATURE_MS1 (FEATURE_ID);
-CREATE INDEX IF NOT EXISTS idx_feature_ms2_feature_id ON FEATURE_MS2 (FEATURE_ID);
-CREATE INDEX IF NOT EXISTS idx_score_ms1_feature_id ON SCORE_MS1 (FEATURE_ID);
-CREATE INDEX IF NOT EXISTS idx_score_ms2_feature_id ON SCORE_MS2 (FEATURE_ID);
-CREATE INDEX IF NOT EXISTS idx_score_ipf_feature_id ON SCORE_IPF (FEATURE_ID);
-
-CREATE INDEX IF NOT EXISTS idx_score_ipf_peptide_id ON SCORE_IPF (PEPTIDE_ID);
 '''
+        if check_sqlite_table(con, "FEATURE_MS1"):
+          idx_query += "CREATE INDEX IF NOT EXISTS idx_feature_ms1_feature_id ON FEATURE_MS1 (FEATURE_ID);"
+        if check_sqlite_table(con, "FEATURE_MS2"):
+          idx_query += "CREATE INDEX IF NOT EXISTS idx_feature_ms2_feature_id ON FEATURE_MS2 (FEATURE_ID);"
+        if check_sqlite_table(con, "SCORE_MS1"):
+          idx_query += "CREATE INDEX IF NOT EXISTS idx_score_ms1_feature_id ON SCORE_MS1 (FEATURE_ID);"
+          score_ms1_pep = "SCORE_MS1.PEP"
+          link_ms1 = "LEFT JOIN SCORE_MS1 ON SCORE_MS1.FEATURE_ID = FEATURE.ID"
+        else:
+          score_ms1_pep = "NULL"
+          link_ms1 = ""
+        if check_sqlite_table(con, "SCORE_MS2"):
+          idx_query += "CREATE INDEX IF NOT EXISTS idx_score_ms2_feature_id ON SCORE_MS2 (FEATURE_ID);"
+        if check_sqlite_table(con, "SCORE_IPF"):
+          idx_query += "CREATE INDEX IF NOT EXISTS idx_score_ipf_feature_id ON SCORE_IPF (FEATURE_ID);"
+          idx_query += "CREATE INDEX IF NOT EXISTS idx_score_ipf_peptide_id ON SCORE_IPF (PEPTIDE_ID);"
+
         query = '''
 SELECT RUN.ID AS id_run,
        PEPTIDE.ID AS id_peptide,
@@ -61,7 +71,7 @@ SELECT RUN.ID AS id_run,
        FEATURE_MS1.APEX_INTENSITY AS aggr_prec_Peak_Apex,
        FEATURE.LEFT_WIDTH AS leftWidth,
        FEATURE.RIGHT_WIDTH AS rightWidth,
-       SCORE_MS1.PEP AS ms1_pep,
+       %s AS ms1_pep,
        SCORE_MS2.PEP AS ms2_pep,
        SCORE_IPF.PRECURSOR_PEAKGROUP_PEP AS precursor_pep,
        SCORE_IPF.PEP AS ipf_pep,
@@ -76,14 +86,14 @@ INNER JOIN FEATURE ON FEATURE.PRECURSOR_ID = PRECURSOR.ID
 INNER JOIN RUN ON RUN.ID = FEATURE.RUN_ID
 LEFT JOIN FEATURE_MS1 ON FEATURE_MS1.FEATURE_ID = FEATURE.ID
 LEFT JOIN FEATURE_MS2 ON FEATURE_MS2.FEATURE_ID = FEATURE.ID
-LEFT JOIN SCORE_MS1 ON SCORE_MS1.FEATURE_ID = FEATURE.ID
+%s
 LEFT JOIN SCORE_MS2 ON SCORE_MS2.FEATURE_ID = FEATURE.ID
 LEFT JOIN SCORE_IPF ON SCORE_IPF.FEATURE_ID = FEATURE.ID
 INNER JOIN PEPTIDE AS PEPTIDE_IPF ON SCORE_IPF.PEPTIDE_ID = PEPTIDE_IPF.ID
 WHERE SCORE_MS2.QVALUE < %s AND SCORE_IPF.PEP < %s
 ORDER BY transition_group_id,
          peak_group_rank;
-''' % (max_rs_peakgroup_qvalue, ipf_max_peptidoform_pep)
+''' % (score_ms1_pep, link_ms1, max_rs_peakgroup_qvalue, ipf_max_peptidoform_pep)
     # Main query for augmented IPF
     elif ipf_present and ipf=='augmented':
         idx_query = '''
@@ -98,14 +108,24 @@ CREATE INDEX IF NOT EXISTS idx_run_run_id ON RUN (ID);
 CREATE INDEX IF NOT EXISTS idx_feature_run_id ON FEATURE (RUN_ID);
 
 CREATE INDEX IF NOT EXISTS idx_feature_feature_id ON FEATURE (ID);
-CREATE INDEX IF NOT EXISTS idx_feature_ms1_feature_id ON FEATURE_MS1 (FEATURE_ID);
-CREATE INDEX IF NOT EXISTS idx_feature_ms2_feature_id ON FEATURE_MS2 (FEATURE_ID);
-CREATE INDEX IF NOT EXISTS idx_score_ms1_feature_id ON SCORE_MS1 (FEATURE_ID);
-CREATE INDEX IF NOT EXISTS idx_score_ms2_feature_id ON SCORE_MS2 (FEATURE_ID);
-CREATE INDEX IF NOT EXISTS idx_score_ipf_feature_id ON SCORE_IPF (FEATURE_ID);
-
-CREATE INDEX IF NOT EXISTS idx_score_ipf_peptide_id ON SCORE_IPF (PEPTIDE_ID);
 '''
+        if check_sqlite_table(con, "FEATURE_MS1"):
+          idx_query += "CREATE INDEX IF NOT EXISTS idx_feature_ms1_feature_id ON FEATURE_MS1 (FEATURE_ID);"
+        if check_sqlite_table(con, "FEATURE_MS2"):
+          idx_query += "CREATE INDEX IF NOT EXISTS idx_feature_ms2_feature_id ON FEATURE_MS2 (FEATURE_ID);"
+        if check_sqlite_table(con, "SCORE_MS1"):
+          idx_query += "CREATE INDEX IF NOT EXISTS idx_score_ms1_feature_id ON SCORE_MS1 (FEATURE_ID);"
+          score_ms1_pep = "SCORE_MS1.PEP"
+          link_ms1 = "LEFT JOIN SCORE_MS1 ON SCORE_MS1.FEATURE_ID = FEATURE.ID"
+        else:
+          score_ms1_pep = "NULL"
+          link_ms1 = ""
+        if check_sqlite_table(con, "SCORE_MS2"):
+          idx_query += "CREATE INDEX IF NOT EXISTS idx_score_ms2_feature_id ON SCORE_MS2 (FEATURE_ID);"
+        if check_sqlite_table(con, "SCORE_IPF"):
+          idx_query += "CREATE INDEX IF NOT EXISTS idx_score_ipf_feature_id ON SCORE_IPF (FEATURE_ID);"
+          idx_query += "CREATE INDEX IF NOT EXISTS idx_score_ipf_peptide_id ON SCORE_IPF (PEPTIDE_ID);"
+
         query = '''
 SELECT RUN.ID AS id_run,
        PEPTIDE.ID AS id_peptide,
@@ -132,7 +152,7 @@ SELECT RUN.ID AS id_run,
        SCORE_MS2.RANK AS peak_group_rank,
        SCORE_MS2.SCORE AS d_score,
        SCORE_MS2.QVALUE AS m_score,
-       SCORE_MS1.PEP AS ms1_pep,
+       %s AS ms1_pep,
        SCORE_MS2.PEP AS ms2_pep
 FROM PRECURSOR
 INNER JOIN PRECURSOR_PEPTIDE_MAPPING ON PRECURSOR.ID = PRECURSOR_PEPTIDE_MAPPING.PRECURSOR_ID
@@ -141,12 +161,12 @@ INNER JOIN FEATURE ON FEATURE.PRECURSOR_ID = PRECURSOR.ID
 INNER JOIN RUN ON RUN.ID = FEATURE.RUN_ID
 LEFT JOIN FEATURE_MS1 ON FEATURE_MS1.FEATURE_ID = FEATURE.ID
 LEFT JOIN FEATURE_MS2 ON FEATURE_MS2.FEATURE_ID = FEATURE.ID
-LEFT JOIN SCORE_MS1 ON SCORE_MS1.FEATURE_ID = FEATURE.ID
+%s
 LEFT JOIN SCORE_MS2 ON SCORE_MS2.FEATURE_ID = FEATURE.ID
 WHERE SCORE_MS2.QVALUE < %s
 ORDER BY transition_group_id,
          peak_group_rank;
-''' % max_rs_peakgroup_qvalue
+''' % (score_ms1_pep, link_ms1, max_rs_peakgroup_qvalue)
         query_augmented = '''
 SELECT FEATURE_ID AS id,
        MODIFIED_SEQUENCE AS ipf_FullUniModPeptideName,
@@ -171,10 +191,14 @@ CREATE INDEX IF NOT EXISTS idx_run_run_id ON RUN (ID);
 CREATE INDEX IF NOT EXISTS idx_feature_run_id ON FEATURE (RUN_ID);
 
 CREATE INDEX IF NOT EXISTS idx_feature_feature_id ON FEATURE (ID);
-CREATE INDEX IF NOT EXISTS idx_feature_ms1_feature_id ON FEATURE_MS1 (FEATURE_ID);
-CREATE INDEX IF NOT EXISTS idx_feature_ms2_feature_id ON FEATURE_MS2 (FEATURE_ID);
-CREATE INDEX IF NOT EXISTS idx_score_ms2_feature_id ON SCORE_MS2 (FEATURE_ID);
 '''
+        if check_sqlite_table(con, "FEATURE_MS1"):
+          idx_query += "CREATE INDEX IF NOT EXISTS idx_feature_ms1_feature_id ON FEATURE_MS1 (FEATURE_ID);"
+        if check_sqlite_table(con, "FEATURE_MS2"):
+          idx_query += "CREATE INDEX IF NOT EXISTS idx_feature_ms2_feature_id ON FEATURE_MS2 (FEATURE_ID);"
+        if check_sqlite_table(con, "SCORE_MS2"):
+          idx_query += "CREATE INDEX IF NOT EXISTS idx_score_ms2_feature_id ON SCORE_MS2 (FEATURE_ID);"
+
         query = '''
 SELECT RUN.ID AS id_run,
        PEPTIDE.ID AS id_peptide,
@@ -423,8 +447,16 @@ INNER JOIN
    FROM FEATURE) AS FEATURE ON FEATURE_MS2.FEATURE_ID = FEATURE.ID
 INNER JOIN
   (SELECT ID,
+          CHARGE AS VAR_PRECURSOR_CHARGE,
           DECOY
    FROM PRECURSOR) AS PRECURSOR ON FEATURE.PRECURSOR_ID = PRECURSOR.ID
+INNER JOIN
+  (SELECT PRECURSOR_ID AS ID,
+          COUNT(*) AS VAR_TRANSITION_NUM_SCORE
+   FROM TRANSITION_PRECURSOR_MAPPING
+   INNER JOIN TRANSITION ON TRANSITION_PRECURSOR_MAPPING.TRANSITION_ID = TRANSITION.ID
+   WHERE DETECTING==1
+   GROUP BY PRECURSOR_ID) AS VAR_TRANSITION_SCORE ON FEATURE.PRECURSOR_ID = VAR_TRANSITION_SCORE.ID
 INNER JOIN SCORE_MS2 ON FEATURE.ID = SCORE_MS2.FEATURE_ID
 WHERE RANK == 1
 ORDER BY RUN_ID,
@@ -447,6 +479,7 @@ INNER JOIN
    FROM FEATURE) AS FEATURE ON FEATURE_MS1.FEATURE_ID = FEATURE.ID
 INNER JOIN
   (SELECT ID,
+          CHARGE AS VAR_PRECURSOR_CHARGE,
           DECOY
    FROM PRECURSOR) AS PRECURSOR ON FEATURE.PRECURSOR_ID = PRECURSOR.ID
 INNER JOIN SCORE_MS1 ON FEATURE.ID = SCORE_MS1.FEATURE_ID
@@ -462,6 +495,8 @@ ORDER BY RUN_ID,
         table_transition = pd.read_sql_query('''
 SELECT TRANSITION.DECOY AS DECOY,
        FEATURE_TRANSITION.*,
+       PRECURSOR.CHARGE AS VAR_PRECURSOR_CHARGE,
+       TRANSITION.VAR_PRODUCT_CHARGE AS VAR_PRODUCT_CHARGE,
        SCORE_TRANSITION.*,
        RUN_ID || '_' || FEATURE_TRANSITION.FEATURE_ID || '_' || PRECURSOR_ID || '_' || FEATURE_TRANSITION.TRANSITION_ID AS GROUP_ID
 FROM FEATURE_TRANSITION
@@ -476,6 +511,7 @@ INNER JOIN SCORE_TRANSITION ON FEATURE_TRANSITION.FEATURE_ID = SCORE_TRANSITION.
 AND FEATURE_TRANSITION.TRANSITION_ID = SCORE_TRANSITION.TRANSITION_ID
 INNER JOIN
   (SELECT ID,
+          CHARGE AS VAR_PRODUCT_CHARGE,
           DECOY
    FROM TRANSITION) AS TRANSITION ON FEATURE_TRANSITION.TRANSITION_ID = TRANSITION.ID
 ORDER BY RUN_ID,
